@@ -33,17 +33,17 @@
 
 #ifndef	lint
 static char *moduleid = 
-	"@(#)$Id: apprentice.c,v 1.16 93/02/19 14:22:42 ian Exp $";
+	"@(#)$Id: apprentice.c,v 1.19 1994/05/03 17:58:23 christos Exp $";
 #endif	/* lint */
 
 #define	EATAB {while (isascii((unsigned char) *l) && \
 		      isspace((unsigned char) *l))  ++l;}
 
 
-static int getvalue __P((struct magic *, char **));
-static int hextoint __P((int));
-static char *getstr __P((char *, char *, int, int *));
-static int parse    __P((char *, int *, int));
+static int getvalue	__P((struct magic *, char **));
+static int hextoint	__P((int));
+static char *getstr	__P((char *, char *, int, int *));
+static int parse	__P((char *, int *, int));
 
 static int maxmagic = 0;
 
@@ -67,7 +67,7 @@ int check;			/* non-zero? checking-only run. */
 	}
 
         maxmagic = MAXMAGIS;
-	if ((magic = (struct magic *) malloc(sizeof(struct magic) * maxmagic))
+	if ((magic = (struct magic *) calloc(sizeof(struct magic), maxmagic))
 	    == NULL) {
 		(void) fprintf(stderr, "%s: Out of memory.\n", progname);
 		if (check)
@@ -75,7 +75,7 @@ int check;			/* non-zero? checking-only run. */
 		else
 			exit(1);
 	}
-
+  
 	/* parse it */
 	if (check)	/* print silly verbose header for USG compat. */
 		(void) printf("cont\toffset\ttype\topcode\tmask\tvalue\tdesc\n");
@@ -94,6 +94,47 @@ int check;			/* non-zero? checking-only run. */
 	return errs ? -1 : 0;
 }
 
+/*
+ * extend the sign bit if the comparison is to be signed
+ */
+unsigned long
+signextend(m, v)
+struct magic *m;
+unsigned long v;
+{
+	if (!(m->flag & UNSIGNED))
+		switch(m->type) {
+		/*
+		 * Do not remove the casts below.  They are
+		 * vital.  When later compared with the data,
+		 * the sign extension must have happened.
+		 */
+		case BYTE:
+			v = (char) v;
+			break;
+		case SHORT:
+		case BESHORT:
+		case LESHORT:
+			v = (short) v;
+			break;
+		case DATE:
+		case BEDATE:
+		case LEDATE:
+		case LONG:
+		case BELONG:
+		case LELONG:
+			v = (long) v;
+			break;
+		case STRING:
+			break;
+		default:
+			magwarn("can't happen: m->type=%d\n",
+				m->type);
+			return -1;
+		}
+	return v;
+}
+
 /*
  * parse one line from magic file, put into magic[index++] if valid
  */
@@ -164,9 +205,11 @@ int *ndx, check;
 		s = l;
 		if (*l == '+' || *l == '-') l++;
 		if (isdigit((unsigned char)*l)) {
-		    m->in.offset = strtol(l, &t, 0);
-		    if (*s == '-') m->in.offset = - m->in.offset;
+			m->in.offset = strtol(l, &t, 0);
+			if (*s == '-') m->in.offset = - m->in.offset;
 		}
+		else
+			t = l;
 		if (*t++ != ')') 
 			magwarn("missing ')' in indirect offset");
 		l = t;
@@ -189,6 +232,11 @@ int *ndx, check;
 #define NLELONG		6
 #define NLEDATE		6
 
+	if (*l == 'u') {
+		++l;
+		m->flag |= UNSIGNED;
+	}
+
 	/* get type, skip it */
 	if (strncmp(l, "byte", NBYTE)==0) {
 		m->type = BYTE;
@@ -230,9 +278,9 @@ int *ndx, check;
 	/* New-style anding: "0 byte&0x80 =0x80 dynamically linked" */
 	if (*l == '&') {
 		++l;
-		m->mask = strtol(l, &l, 0);
+		m->mask = signextend(m, strtol(l, &l, 0));
 	} else
-		m->mask = 0L;
+		m->mask = ~0L;
 	EATAB;
   
 	switch (*l) {
@@ -311,36 +359,9 @@ char **p;
 	if (m->type == STRING) {
 		*p = getstr(*p, m->value.s, sizeof(m->value.s), &slen);
 		m->vallen = slen;
-	} else {
-		if (m->reln != 'x') {
-			switch(m->type) {
-			/*
-			 * Do not remove the casts below.  They are vital.
-			 * When later compared with the data, the sign
-			 * extension must have happened.
-			 */
-			case BYTE:
-				m->value.l = (char) strtol(*p,p,0);
-				break;
-			case SHORT:
-			case BESHORT:
-			case LESHORT:
-				m->value.l = (short) strtol(*p,p,0);
-				break;
-			case DATE:
-			case BEDATE:
-			case LEDATE:
-			case LONG:
-			case BELONG:
-			case LELONG:
-				m->value.l = (long) strtol(*p,p,0);
-				break;
-			default:
-				magwarn("can't happen: m->type=%d\n", m->type);
-				return -1;
-			}
-		}
-	}
+	} else
+		if (m->reln != 'x')
+			m->value.l = signextend(m, strtol(*p, p, 0));
 	return 0;
 }
 
@@ -474,47 +495,57 @@ int c;
  * Print a string containing C character escapes.
  */
 void
-showstr(s)
+showstr(fp, s, len)
+FILE *fp;
 const char *s;
+int len;
 {
 	register char	c;
 
-	while((c = *s++) != '\0') {
+	for (;;) {
+		c = *s++;
+		if (len == -1) {
+			if (c == '\0')
+				break;
+		}
+		else  {
+			if (len-- == 0)
+				break;
+		}
 		if(c >= 040 && c <= 0176)	/* TODO isprint && !iscntrl */
-			putchar(c);
+			(void) fputc(c, fp);
 		else {
-			putchar('\\');
+			(void) fputc('\\', fp);
 			switch (c) {
 			
 			case '\n':
-				putchar('n');
+				(void) fputc('n', fp);
 				break;
 
 			case '\r':
-				putchar('r');
+				(void) fputc('r', fp);
 				break;
 
 			case '\b':
-				putchar('b');
+				(void) fputc('b', fp);
 				break;
 
 			case '\t':
-				putchar('t');
+				(void) fputc('t', fp);
 				break;
 
 			case '\f':
-				putchar('f');
+				(void) fputc('f', fp);
 				break;
 
 			case '\v':
-				putchar('v');
+				(void) fputc('v', fp);
 				break;
 
 			default:
-				printf("%.3o", c & 0377);
+				(void) fprintf(fp, "%.3o", c & 0377);
 				break;
 			}
 		}
 	}
-	putchar('\t');
 }
