@@ -26,7 +26,7 @@
  */
 #ifndef	lint
 static char *moduleid = 
-	"@(#)$Id: file.c,v 1.26 93/02/19 14:22:45 ian Exp $";
+	"@(#)$Id: file.c,v 1.29 1993/10/27 20:59:05 christos Exp $";
 #endif	/* lint */
 
 #include <stdio.h>
@@ -201,7 +201,8 @@ int wid;
 	}
 
 	if (wid > 0)
-	     (void) printf("%s:%*s ", inname, wid - strlen(inname), "");
+	     (void) printf("%s:%*s ", inname, 
+			   (int) (wid - strlen(inname)), "");
 
 	if (inname != stdname) {
 	    /*
@@ -234,8 +235,8 @@ int wid;
 	if (nbytes == 0) 
 		ckfputs("empty", stdout);
 	else {
-		buf[nbytes] = '\0';	/* null-terminate it */
-		tryit(buf, nbytes);
+		buf[nbytes++] = '\0';	/* null-terminate it */
+		tryit(buf, nbytes, zflag);
 	}
 
 	if (inname != stdname) {
@@ -252,21 +253,24 @@ int wid;
 
 
 void
-tryit(buf, nb)
+tryit(buf, nb, zflag)
 unsigned char *buf;
-int nb;
+int nb, zflag;
 {
 	/*
-	 * try tests in /etc/magic (or surrogate magic file)
+	 * Try compression stuff
 	 */
-	if (softmagic(buf, nb) != 1)
-	    /*
-	     * try known keywords, check for ascii-ness too.
-	     */
-	    if (ascmagic(buf, nb) != 1)
+	if (!zflag || zmagic(buf, nb) != 1)
 		/*
-		 * abandon hope, all ye who remain here
+		 * try tests in /etc/magic (or surrogate magic file)
 		 */
-		ckfputs("data", stdout);
+		if (softmagic(buf, nb) != 1)
+			/*
+			 * try known keywords, check for ascii-ness too.
+			 */
+			if (ascmagic(buf, nb) != 1)
+			    /*
+			     * abandon hope, all ye who remain here
+			     */
+			    ckfputs("data", stdout);
 }
-
