@@ -1,6 +1,6 @@
 /*
  * file.h - definitions for file(1) program
- * @(#)$Id: file.h,v 1.14 93/02/19 14:22:44 ian Exp $
+ * @(#)$Id: file.h,v 1.19 1994/05/03 17:58:23 christos Exp $
  *
  * Copyright (c) Ian F. Darwin, 1987.
  * Written by Ian F. Darwin.
@@ -26,7 +26,7 @@
  * 4. This notice may not be removed or altered.
  */
 
-#define HOWMANY	1024		/* how much of the file to look at */
+#define HOWMANY	8192		/* how much of the file to look at */
 #define MAXMAGIS 1000		/* max entries in /etc/magic */
 #define MAXDESC	50		/* max leng of text description */
 #define MAXstring 32		/* max leng of "string" types */
@@ -34,13 +34,13 @@
 struct magic {
 	short flag;		
 #define INDIR	1		/* if '>(...)' appears,  */
+#define	UNSIGNED 2		/* comparison is unsigned */
 	short cont_level;	/* level of ">" */
 	struct {
 		char type;	/* byte short long */
 		long offset;	/* offset from indirection */
 	} in;
 	long offset;		/* offset to magic number */
-#define	MASK	0200		/* this is a masked op, like & v1 = v2 */
 	unsigned char reln;	/* relation (0=eq, '>'=gt, etc) */
 	char type;		/* int, short, long or string. */
 	char vallen;		/* length of string value, if any */
@@ -56,14 +56,14 @@ struct magic {
 #define				LELONG	11
 #define				LEDATE	12
 	union VALUETYPE {
-		char b;
-		short h;
-		long l;
+		unsigned char b;
+		unsigned short h;
+		unsigned long l;
 		char s[MAXstring];
 		unsigned char hs[2];	/* 2 bytes of a fixed-endian "short" */
 		unsigned char hl[4];	/* 2 bytes of a fixed-endian "long" */
 	} value;		/* either number or string */
-	long mask;		/* mask before comparison with value */
+	unsigned long mask;	/* mask before comparison with value */
 	char nospflag;		/* supress space character */
 	char desc[MAXDESC];	/* description */
 };
@@ -86,15 +86,16 @@ extern void  ckfputs		__P((const char *, FILE *));
 struct stat;
 extern int   fsmagic		__P((const char *, struct stat *));
 extern int   is_compress	__P((const unsigned char *, int *));
-extern int   is_tar		__P((unsigned char *));
+extern int   is_tar		__P((unsigned char *, int));
 extern void  magwarn		__P((const char *, ...));
 extern void  mdump		__P((struct magic *));
 extern void  process		__P((const char *, int));
-extern void  showstr		__P((const char *));
+extern void  showstr		__P((FILE *, const char *, int));
 extern int   softmagic		__P((unsigned char *, int));
-extern void  tryit		__P((unsigned char *, int));
-extern int   uncompress		__P((const unsigned char *, unsigned char **, int));
+extern void  tryit		__P((unsigned char *, int, int));
+extern int   zmagic		__P((unsigned char *, int));
 extern void  ckfprintf		__P((FILE *, const char *, ...));
+extern unsigned long signextend	__P((struct magic *, unsigned long));
 
 
 
@@ -115,7 +116,7 @@ extern int lflag;		/* follow symbolic links?		*/
 extern int optind;		/* From getopt(3)			*/
 extern char *optarg;
 
-#if !defined(__STDC__) || defined(sun)
+#if !defined(__STDC__) || defined(sun) || defined(__sun__) || defined(__convex__)
 extern int sys_nerr;
 extern char *sys_errlist[];
 #define strerror(e) \
