@@ -35,42 +35,76 @@
 #endif
 #include <stdlib.h>
 #include <unistd.h>
+#include <time.h>
 #include "file.h"
 
 #ifndef lint
 static char *moduleid =
-	"@(#)$Id: print.c,v 1.18 93/02/19 14:22:47 ian Exp $";
+	"@(#)$Id: print.c,v 1.21 1994/05/03 17:58:23 christos Exp $";
 #endif  /* lint */
 
+#define SZOF(a)	(sizeof(a) / sizeof(a[0]))
+
 void
 mdump(m)
 struct magic *m;
 {
-	static char *offs[] = {  "absolute", "offset", 
-				 "indirect", "indirect-offset" };
 	static char *typ[] = {   "invalid", "byte", "short", "invalid",
 				 "long", "string", "date", "beshort",
 				 "belong", "bedate", "leshort", "lelong",
 				 "ledate" };
-	(void) fprintf(stderr, "[%s,%d,%s,%s%c,",
-		(m->flag >= 0 && m->flag < 4 ? offs[m->flag]: "*bad*"),
-		m->offset,
-		(m->type >= 0 && m->type < 7 ? 
-				typ[(unsigned char) m->type] : "*bad*"),
-		m->reln & MASK ? "&" : "",
-		m->reln & ~MASK);
-	if (m->flag & INDIR)
-	    (void) fprintf(stderr, "(%s,%d)",
-		(m->in.type >= 0 && 
-		m->in.type < 6 ? typ[(unsigned char) m->in.type] : "*bad*"),
-		m->in.offset);
+	(void) fputc('[', stderr);
+	(void) fprintf(stderr, ">>>>>>>> %d" + 8 - (m->cont_level & 7),
+		       m->offset);
 
-	if (m->type == STRING)
-		showstr(m->value.s);
-	else
-		(void) fprintf(stderr, "%d",m->value.l);
-	(void) fprintf(stderr, ",%s", m->desc);
-	(void) fputs("]\n", stderr);
+	if (m->flag & INDIR)
+		(void) fprintf(stderr, "(%s,%d),",
+			       (m->in.type >= 0 && m->in.type < SZOF(typ)) ? 
+					typ[(unsigned char) m->in.type] :
+					"*bad*",
+			       m->in.offset);
+
+	(void) fprintf(stderr, " %s%s", (m->flag & UNSIGNED) ? "u" : "",
+		       (m->type >= 0 && m->type < SZOF(typ)) ? 
+				typ[(unsigned char) m->type] : 
+				"*bad*");
+	if (m->mask != ~0L)
+		(void) fprintf(stderr, " & %.8x", m->mask);
+
+	(void) fprintf(stderr, ",%c", m->reln);
+
+	if (m->reln != 'x') {
+	    switch (m->type) {
+	    case BYTE:
+	    case SHORT:
+	    case LONG:
+	    case LESHORT:
+	    case LELONG:
+	    case BESHORT:
+	    case BELONG:
+		    (void) fprintf(stderr, "%d", m->value.l);
+		    break;
+	    case STRING:
+		    showstr(stderr, m->value.s, -1);
+		    break;
+	    case DATE:
+	    case LEDATE:
+	    case BEDATE:
+		    {
+			    char *rt, *pp = ctime((time_t*) &m->value.l);
+			    if ((rt = strchr(pp, '\n')) != NULL)
+				    *rt = '\0';
+			    (void) fprintf(stderr, "%s,", pp);
+			    if (rt)
+				    *rt = '\n';
+		    }
+		    break;
+	    default:
+		    (void) fputs("*bad*", stderr);
+		    break;
+	    }
+	}
+	(void) fprintf(stderr, ",\"%s\"]\n", m->desc);
 }
 
 /*
